// Si5351 VFO, CLK0, CLK2, 20m, 8-digit 7-segnemt LED display MAX7219, 50Hz/1000Hz, v0.1
// 25.01.2019, Arduino IDE v1.8.8, LZ2WSG, KN34PC
//---------------------------------------------------------------------------------------------------------
#include "src/Rotary.h"                 // http://www.buxtronix.net/2011/10/rotary-encoders-done-properly.html
#include "src/si5351.h"                 // https://github.com/etherkit/Si5351Arduino, v2.1.2
#include "src/LedControl.h"             // https://github.com/wayoda/LedControl/releases

#define DEFAULT_RX    14060000
#define BAND_L        14000000
#define BAND_H        14350000
#define IF_FREQ       8867000
#define SW_STEP       A0                // encoder step switch
#define ENC_INC_1     50                // encoder increment 1
#define ENC_INC_2     1000              // encoder increment 2

Rotary r = Rotary(2, 3);
Si5351 si5351(0x60);                    // Si5351 I2C address
LedControl lc = LedControl(6, 5, 4, 1); // LED display (DIN, CLK, CS, no. devices)

uint32_t rx = DEFAULT_RX;
uint32_t enc_inc = ENC_INC_2;
int8_t enc_dir = 0;                     // -1 DIR_CCW, 0 DIR_NONE, 1 DIR_CCW
bool ch_flag = true;
bool step_point = false;
//---------------------------------------------------------------------------------------------------------
void setup() {
  pinMode(SW_STEP, INPUT_PULLUP);

  attachInterrupt(0, rotary_encoder, CHANGE);
  attachInterrupt(1, rotary_encoder, CHANGE);

  si5351.init(SI5351_CRYSTAL_LOAD_8PF, 0, 7500);  // crystal 25.000 MHz, my correction is 7500 ???
  // si5351.set_freq(IF_FREQ * SI5351_FREQ_MULT, SI5351_CLK2);  // Fif, enable/disable CLK2

  lc.shutdown(0, false);
  lc.setIntensity(0, 10);
  lc.clearDisplay(0);
}
//---------------------------------------------------------------------------------------------------------
void loop() {
  if (ch_flag) {
    ch_flag = false;
    rx += enc_dir * enc_inc;
    enc_dir = 0;
    rx = constrain(rx, BAND_L, BAND_H);

    si5351_set_freq(rx - IF_FREQ);      // Fhet = Fin - Fif
    // si5351_set_freq(rx + IF_FREQ);   // Fhet = Fin + Fif

    show_freq(rx);
  }

  if (digitalRead(SW_STEP) == LOW) {    // increment events
    if (button_state(SW_STEP) > 0) {
      set_inc();
      show_inc();
      do {
      }
      while (digitalRead(SW_STEP) == LOW);
    }
  }
}
//---------------------------------------------------------------------------------------------------------
void rotary_encoder() {                 // rotary encoder events
  uint8_t result = r.process();

  if (result) {
    ch_flag = true;
    if (result == DIR_CW)
      enc_dir = 1;
    else
      enc_dir = -1;
  }
}
//---------------------------------------------------------------------------------------------------------
void si5351_set_freq(uint32_t frequency) {
  si5351.set_freq(frequency * SI5351_FREQ_MULT, SI5351_CLK0);
}
//---------------------------------------------------------------------------------------------------------
void set_inc() {
  if (enc_inc == ENC_INC_1) {
    enc_inc = ENC_INC_2;                // 1000 Hz, round to XX.XXX.000
    float tmp = round (rx / (ENC_INC_2 * 1.000));
    rx = (uint32_t)(tmp * ENC_INC_2);
    ch_flag = true;
    step_point = false;
  }
  else {
    enc_inc = ENC_INC_1;                // 50 Hz
    step_point = true;
  }
}
//---------------------------------------------------------------------------------------------------------
void show_inc() {
  uint8_t x010  = (rx / 10) % 10;

  lc.setDigit(0, 0, x010, step_point);
}
//---------------------------------------------------------------------------------------------------------
void show_freq(uint32_t sf_rx) {
  uint8_t x010m = sf_rx / 10000000;
  uint8_t x001m = (sf_rx / 1000000) % 10;
  uint8_t x100k = (sf_rx / 100000) % 10;
  uint8_t x010k = (sf_rx / 10000) % 10;
  uint8_t x001k = (sf_rx / 1000) % 10;
  uint8_t x100  = (sf_rx / 100) % 10;
  uint8_t x010  = (sf_rx / 10) % 10;

  if (x010m > 0)
    lc.setDigit(0, 6, x010m, false);
  lc.setDigit(0, 5, x001m, true);
  lc.setDigit(0, 4, x100k, false);
  lc.setDigit(0, 3, x010k, false);
  lc.setDigit(0, 2, x001k, true);
  lc.setDigit(0, 1, x100, false);
  lc.setDigit(0, 0, x010, step_point);
}
//---------------------------------------------------------------------------------------------------------
uint8_t button_state(uint8_t pin_bt) {  // push button state: 0(very fast), 1(fast), 2(slow)
  uint32_t t_start;
  uint32_t t_bt = 0;
  uint8_t bt_state;

  t_start = millis();
  while (!digitalRead(pin_bt)) {
    t_bt = millis() - t_start;
    if (t_bt > 1000)
      break;
  }
  if (t_bt < 20)                        // 'nothing' for debounce
    bt_state = 0;
  else if (t_bt < 400)                  // fast
    bt_state = 1;
  else                                  // slow
    bt_state = 2;
  return bt_state;
}
